'use strict';

const node_path = require('node:path');
const core = require('@babel/core');
const helperPluginUtils = require('@babel/helper-plugin-utils');

const index = helperPluginUtils.declare((_, options)=>{
    return {
        name: "babel-plugin-inject",
        visitor: {
            Identifier (path, { filename = "" }) {
                const identifier = path.node.name;
                if (!Object.hasOwn(options, identifier) || path.scope.hasBinding(identifier)) {
                    return;
                }
                const program = path.getAncestry().find((path)=>path.isProgram());
                if (!program) {
                    throw path.buildCodeFrameError(`Cannot find program node`);
                }
                const [source, name] = options[identifier];
                let normalizedSource = source;
                if (normalizedSource[0] === "." || node_path.isAbsolute(normalizedSource)) {
                    normalizedSource = node_path.relative(node_path.dirname(filename), normalizedSource);
                    normalizedSource = normalizedSource[0] === "." ? normalizedSource : `./${normalizedSource}`;
                }
                const [importDeclaration] = program.unshiftContainer("body", core.types.importDeclaration([
                    core.types.importSpecifier(core.types.identifier(identifier), core.types.identifier(name))
                ], core.types.stringLiteral(normalizedSource)));
                program.scope.registerDeclaration(importDeclaration);
            }
        }
    };
});

module.exports = index;
