import { isAbsolute, relative, dirname } from 'node:path';
import { types } from '@babel/core';
import { declare } from '@babel/helper-plugin-utils';

const index = declare((_, options)=>{
    return {
        name: "babel-plugin-inject",
        visitor: {
            Identifier (path, { filename = "" }) {
                const identifier = path.node.name;
                if (!Object.hasOwn(options, identifier) || path.scope.hasBinding(identifier)) {
                    return;
                }
                const program = path.getAncestry().find((path)=>path.isProgram());
                if (!program) {
                    throw path.buildCodeFrameError(`Cannot find program node`);
                }
                const [source, name] = options[identifier];
                let normalizedSource = source;
                if (normalizedSource[0] === "." || isAbsolute(normalizedSource)) {
                    normalizedSource = relative(dirname(filename), normalizedSource);
                    normalizedSource = normalizedSource[0] === "." ? normalizedSource : `./${normalizedSource}`;
                }
                const [importDeclaration] = program.unshiftContainer("body", types.importDeclaration([
                    types.importSpecifier(types.identifier(identifier), types.identifier(name))
                ], types.stringLiteral(normalizedSource)));
                program.scope.registerDeclaration(importDeclaration);
            }
        }
    };
});

export { index as default };
